/**
 * Instrumentation Vuze Plugin
 * 
 * Copyright 2008, Yale Laboratory of Networked Systems
 */

package net.yale.cs.p4p.p2p.azureus.log;

import java.net.MalformedURLException;
import java.net.URL;

import org.gudy.azureus2.plugins.ui.model.BasicPluginConfigModel;

public class Config
{

	/* Resource names for strings displayed in UI */
	private static final String RESOURCE_NAMES_PATH = "net.yale.cs.p4p.p2p.azureus.log.Messages";
	private static final String RESOURCE_NAME_DISPLAY = "yale.log.PluginName";
	private static final String RESOURCE_NAME_CFG_ENABLED = "yale.log.Enabled";
	private static final String RESOURCE_NAME_CFG_PORTAL_DB_URL = "yale.log.PortalDBURL";
	private static final String RESOURCE_NAME_CFG_PORTAL_DB_INTERVAL = "yale.log.PortalDBInterval";
	private static final String RESOURCE_NAME_CFG_LOG_POSTED_RECORDS = "yale.log.LogPostedRecords";
	private static final String RESOURCE_NAME_CFG_LOG_SERVER_URL = "yale.log.LogServerURL";
	private static final String RESOURCE_NAME_CFG_HASH_KEEP_BYTES = "yale.log.HashKeepBytes";
	private static final String RESOURCE_NAME_CFG_ADDR_KEEP_BITS_V4 = "yale.log.AddrKeepBitsIPV4";
	private static final String RESOURCE_NAME_CFG_ADDR_KEEP_BITS_V6 = "yale.log.AddrKeepBitsIPV6";
	private static final String RESOURCE_NAME_CFG_LOG_POST_INTERVAL = "yale.log.LogPostInterval";
	private static final String RESOURCE_NAME_CFG_READ_TIMEOUT = "yale.log.ReadTimeout";
	private static final String RESOURCE_NAME_CFG_CONNECT_TIMEOUT = "yale.log.ConnectTimeout";

	private static final String CFG_ENABLED_KEY = "ENABLED";
	private static final boolean CFG_ENABLED_DEFAULT = true;
	public boolean getCfgEnabled() {
		return plugin_.getPluginInterface()
			.getPluginconfig()
			.getPluginBooleanParameter(CFG_ENABLED_KEY, CFG_ENABLED_DEFAULT);
	}

	private static final String CFG_PORTAL_DB_URL_KEY = "PORTAL_DB_URL";
	private static final String CFG_PORTAL_DB_URL_DEFAULT = "http://p4p-6.cs.yale.edu:6670/portal";
	public URL getCfgPortalDBURL() throws MalformedURLException {
		return new URL(
			plugin_.getPluginInterface()
				.getPluginconfig()
				.getPluginStringParameter(CFG_PORTAL_DB_URL_KEY, CFG_PORTAL_DB_URL_DEFAULT));
	}

	private static final String CFG_PORTAL_DB_INTERVAL_KEY = "PORTAL_DB_INTERVAL";
	private static final int CFG_PORTAL_DB_INTERVAL_DEFAULT = 30 * 60; /* 30 minutes */
	public int getCfgPortalDBInterval() {
		return plugin_.getPluginInterface()
			.getPluginconfig()
			.getPluginIntParameter(CFG_PORTAL_DB_INTERVAL_KEY, CFG_PORTAL_DB_INTERVAL_DEFAULT);
	}

	private static final String CFG_LOG_POSTED_RECORDS_KEY = "LOG_POSTED_RECORDS";
	private static final boolean CFG_LOG_POSTED_RECORDS_DEFAULT = false;
	public boolean getCfgLogPostedRecords() {
		return plugin_.getPluginInterface()
			.getPluginconfig()
			.getPluginBooleanParameter(CFG_LOG_POSTED_RECORDS_KEY, CFG_LOG_POSTED_RECORDS_DEFAULT);
	}

	private static final String CFG_LOG_SERVER_URL_KEY = "LOG_SERVER_URL";
	private static final String CFG_LOG_SERVER_URL_DEFAULT = "http://p4p-2.cs.yale.edu:6675/logs";
	public URL getCfgLogServerURL() throws MalformedURLException {
		return new URL(
			plugin_.getPluginInterface()
			.getPluginconfig()
			.getPluginStringParameter(CFG_LOG_SERVER_URL_KEY, CFG_LOG_SERVER_URL_DEFAULT));
	}

	private static final String CFG_HASH_KEEP_BYTES_KEY = "HASH_KEEP_BYTES";
	private static final int CFG_HASH_KEEP_BYTES_DEFAULT = 4;
	public int getCfgHashKeepBytes() {
		return plugin_.getPluginInterface()
			.getPluginconfig()
			.getPluginIntParameter(CFG_HASH_KEEP_BYTES_KEY, CFG_HASH_KEEP_BYTES_DEFAULT);
	}
	
	private static final String CFG_ADDR_KEEP_BITS_V4_KEY = "ADDR_KEEP_BITS_IPV4";
	private static final int CFG_ADDR_KEEP_BITS_V4_DEFAULT = 29;
	public int getCfgAddrKeepBitsV4() {
		return plugin_.getPluginInterface()
			.getPluginconfig()
			.getPluginIntParameter(CFG_ADDR_KEEP_BITS_V4_KEY, CFG_ADDR_KEEP_BITS_V4_DEFAULT);
	}

	private static final String CFG_ADDR_KEEP_BITS_V6_KEY = "ADDR_KEEP_BITS_IPV6";
	private static final int CFG_ADDR_KEEP_BITS_V6_DEFAULT = 128;
	public int getCfgAddrKeepBitsV6() {
		return plugin_.getPluginInterface()
			.getPluginconfig()
			.getPluginIntParameter(CFG_ADDR_KEEP_BITS_V6_KEY, CFG_ADDR_KEEP_BITS_V6_DEFAULT);
	}

	private static final String CFG_LOG_POST_INTERVAL_KEY = "LOG_POST_INTERVAL";
	private static final int CFG_LOG_POST_INTERVAL_DEFAULT = 30;
	public int getCfgLogPostInterval() {
		return plugin_.getPluginInterface()
			.getPluginconfig()
			.getPluginIntParameter(CFG_LOG_POST_INTERVAL_KEY, CFG_LOG_POST_INTERVAL_DEFAULT);
	}

	private static final String CFG_READ_TIMEOUT_KEY = "READ_TIMEOUT";
	private static final int CFG_READ_TIMEOUT_DEFAULT = 10;
	public int getCfgReadTimeout() {
		return plugin_.getPluginInterface()
			.getPluginconfig()
			.getPluginIntParameter(CFG_READ_TIMEOUT_KEY, CFG_READ_TIMEOUT_DEFAULT);
	}

	private static final String CFG_CONNECT_TIMEOUT_KEY = "CONNECT_TIMEOUT";
	private static final int CFG_CONNECT_TIMEOUT_DEFAULT = 5;
	public int getCfgConnectTimeout() {
		return plugin_.getPluginInterface()
			.getPluginconfig()
			.getPluginIntParameter(CFG_CONNECT_TIMEOUT_KEY, CFG_CONNECT_TIMEOUT_DEFAULT);
	}

	private XFerLogPlugin plugin_;
	private BasicPluginConfigModel cfgView_;

	public Config(XFerLogPlugin plugin) {
		plugin_ = plugin;
		
		/* Install our resource names */
		plugin_.getPluginInterface().getUtilities().getLocaleUtilities().integrateLocalisedMessageBundle(RESOURCE_NAMES_PATH);

		/* Setup our config parameters in the UI */
		cfgView_ = plugin_.getPluginInterface().getUIManager().createBasicPluginConfigModel(RESOURCE_NAME_DISPLAY);
		cfgView_.addBooleanParameter2(CFG_ENABLED_KEY, RESOURCE_NAME_CFG_ENABLED, CFG_ENABLED_DEFAULT);
		/* NOTE: Many options are purposefully disabled from the GUI for now */
		/*
		cfgView_.addStringParameter2(CFG_PORTAL_DB_URL_KEY, RESOURCE_NAME_CFG_PORTAL_DB_URL, CFG_PORTAL_DB_URL_DEFAULT);
		cfgView_.addIntParameter2(CFG_PORTAL_DB_INTERVAL_KEY, RESOURCE_NAME_CFG_PORTAL_DB_INTERVAL, CFG_PORTAL_DB_INTERVAL_DEFAULT);
		cfgView_.addBooleanParameter2(CFG_LOG_POSTED_RECORDS_KEY, RESOURCE_NAME_CFG_LOG_POSTED_RECORDS, CFG_LOG_POSTED_RECORDS_DEFAULT);
		cfgView_.addStringParameter2(CFG_LOG_SERVER_URL_KEY, RESOURCE_NAME_CFG_LOG_SERVER_URL, CFG_LOG_SERVER_URL_DEFAULT);
		cfgView_.addIntParameter2(CFG_HASH_KEEP_BYTES_KEY, RESOURCE_NAME_CFG_HASH_KEEP_BYTES, CFG_HASH_KEEP_BYTES_DEFAULT);
		cfgView_.addIntParameter2(CFG_ADDR_KEEP_BITS_V4_KEY, RESOURCE_NAME_CFG_ADDR_KEEP_BITS_V4, CFG_ADDR_KEEP_BITS_V4_DEFAULT);
		cfgView_.addIntParameter2(CFG_ADDR_KEEP_BITS_V6_KEY, RESOURCE_NAME_CFG_ADDR_KEEP_BITS_V6, CFG_ADDR_KEEP_BITS_V6_DEFAULT);
		cfgView_.addIntParameter2(CFG_LOG_POST_INTERVAL_KEY, RESOURCE_NAME_CFG_LOG_POST_INTERVAL, CFG_LOG_POST_INTERVAL_DEFAULT);
		cfgView_.addIntParameter2(CFG_READ_TIMEOUT_KEY, RESOURCE_NAME_CFG_READ_TIMEOUT, CFG_READ_TIMEOUT_DEFAULT);
		cfgView_.addIntParameter2(CFG_CONNECT_TIMEOUT_KEY, RESOURCE_NAME_CFG_CONNECT_TIMEOUT, CFG_CONNECT_TIMEOUT_DEFAULT);
		*/
	}

	public void cleanup() {
		cfgView_.destroy();
	}
}
