/**
 * Instrumentation Vuze Plugin
 * 
 * Copyright 2008, Yale Laboratory of Networked Systems
 */

package net.yale.cs.p4p.p2p.azureus.log.records;

import org.gudy.azureus2.plugins.peers.Peer;
import org.gudy.azureus2.plugins.peers.PeerStats;

public class PeerTransferStats {

	private long startTime_ = 0;
	public final long getStartTime() { return startTime_; }

	private long endTime_ = 0;
	public final long getEndTime() { return endTime_; }

	private long bytesReceivedStart_ = 0;
	private long bytesReceivedCur_ = 0;
	public final long getBytesReceived() { return bytesReceivedCur_ - bytesReceivedStart_; }

	private long bytesSentStart_ = 0;
	private long bytesSentCur_ = 0;
	public final long getBytesSent() { return bytesSentCur_ - bytesSentStart_; }

	private long bytesDiscardedStart_ = 0;
	private long bytesDiscardedCur_ = 0;
	public final long getBytesDiscarded() { return bytesDiscardedCur_ - bytesDiscardedStart_; }

	private long maxPeerDownloadRate_ = 0;
	public final long getMaxPeerDownloadRate() { return maxPeerDownloadRate_; }

	private long maxPeerUploadRate_ = 0;
	public final long getMaxPeerUploadRate() { return maxPeerUploadRate_; }

	public PeerTransferStats(Peer peer) {
		PeerStats stats = peer.getStats();
		reset(peer, stats);
		update(peer, stats);
	}

	public boolean hasUploadData() {
		return getBytesSent() != 0;
	}

	public boolean hasDownloadData() {
		return getBytesReceived() != 0
			|| getBytesDiscarded() != 0;
	}

	private void reset(Peer peer, PeerStats stats) {
		startTime_ = endTime_ = System.currentTimeMillis();
		bytesReceivedStart_ = bytesReceivedCur_ = stats.getTotalReceived();
		bytesSentStart_ = bytesSentCur_ = stats.getTotalSent();
		bytesDiscardedStart_ = bytesDiscardedCur_ = stats.getTotalDiscarded();
		maxPeerDownloadRate_ = 0;
		maxPeerUploadRate_ = 0;
	}

	public void reset(Peer peer) {
		reset(peer, peer.getStats());
	}

	public void next() {
		startTime_ = endTime_ = System.currentTimeMillis();
		bytesReceivedStart_ = bytesReceivedCur_;
		bytesSentStart_ = bytesSentCur_;
		bytesDiscardedStart_ = bytesDiscardedCur_;
		maxPeerDownloadRate_ = 0;
		maxPeerUploadRate_ = 0;
	}

	private void update(Peer peer, PeerStats stats) {
		endTime_ = System.currentTimeMillis();
		bytesReceivedCur_ = stats.getTotalReceived();
		bytesSentCur_ = stats.getTotalSent();
		bytesDiscardedCur_ = stats.getTotalDiscarded();
		maxPeerDownloadRate_ = Math.max(maxPeerDownloadRate_, stats.getUploadAverage());
		maxPeerUploadRate_ = Math.max(maxPeerUploadRate_, stats.getDownloadAverage());
	}

	public void update(Peer peer) {
		update(peer, peer.getStats());
	}

}
